@@ -3,8 +3,8 @@
  *  XVID MPEG-4 VIDEO CODEC
  *  - Xvid Decoder part of the DShow Filter  -
  *
- *  Copyright(C) 2002-2010 Peter Ross <pross@xvid.org>
- *               2003-2010 Michael Militzer <michael@xvid.org>
+ *  Copyright(C) 2002-2011 Peter Ross <pross@xvid.org>
+ *               2003-2012 Michael Militzer <michael@xvid.org>
  *
  *  This program is free software ; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  *  along with this program ; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * $Id: CXvidDecoder.cpp,v 1.25.2.4 2011/03/17 15:13:25 Isibaar Exp $
+ * $Id: CXvidDecoder.cpp 2156 2017-01-27 13:43:32Z Isibaar $
  *
  ****************************************************************************/
 
@@ -36,10 +36,9 @@
 	C:\DX90SDK\Samples\C++\DirectShow\BaseClasses\Debug
 */
 
-/*
+#ifdef ENABLE_MFT
 #define XVID_USE_MFT
-#define XVID_USE_TRAYICON
-*/
+#endif
 
 #include <windows.h>
 
@@ -176,11 +175,11 @@ CFactoryTemplate g_Templates[] =
 /* note: g_cTemplates must be global; used by strmbase.lib(dllentry.cpp,dllsetup.cpp) */
 int g_cTemplates = sizeof(g_Templates) / sizeof(CFactoryTemplate);
 
-#ifdef XVID_USE_TRAYICON
 extern HINSTANCE g_xvid_hInst;
 
 static int GUI_Page = 0;
-static int Tray_Icon = 0;
+static HWND MSG_hwnd = NULL; /* message handler window */
+
 extern "C" void CALLBACK Configure(HWND hWndParent, HINSTANCE hInstParent, LPSTR lpCmdLine, int nCmdShow );
 
 LRESULT CALLBACK msg_proc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
@@ -190,6 +189,7 @@ LRESULT CALLBACK msg_proc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 	case WM_ICONMESSAGE:
 		switch(lParam)
 		{
+		case WM_LBUTTONUP:
 		case WM_LBUTTONDBLCLK:
 			if (!GUI_Page) {
 				GUI_Page = 1;
@@ -204,21 +204,18 @@ LRESULT CALLBACK msg_proc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 
 	case WM_DESTROY:
 		NOTIFYICONDATA nid;
-		ZeroMemory(&nid,sizeof(NOTIFYICONDATA));
+		ZeroMemory(&nid, sizeof(NOTIFYICONDATA));
 
-		nid.cbSize = NOTIFYICONDATA_V1_SIZE;
+		nid.cbSize = sizeof(NOTIFYICONDATA);
 		nid.hWnd = hwnd;
 		nid.uID = 1456;
-	
 		Shell_NotifyIcon(NIM_DELETE, &nid);
-		Tray_Icon = 0;
 	default:
 		return DefWindowProc(hwnd, uMsg, wParam, lParam);
 	}
 	
 	return TRUE; /* ok */
 }
-#endif
 
 STDAPI DllRegisterServer()
 {
@@ -243,7 +240,7 @@ STDAPI DllRegisterServer()
 	/* Register the MFT decoder */
 	MFTRegister(CLSID_XVID,                          // CLSID
 		        MFT_CATEGORY_VIDEO_DECODER,          // Category
-		        const_cast<LPWSTR>(XVID_NAME_L),     // Friendly name
+				const_cast<LPWSTR>(XVID_NAME_MFT_L), // Friendly name
 		        0,                                   // Flags
 		        inputs_num,                          // Number of input types
 		        mft_bs,                              // Input types
@@ -312,8 +309,6 @@ STDMETHODIMP CXvidDecoder::NonDelegatingQueryInterface(REFIID riid, void **ppv)
 
 /* constructor */
 
-#define XVID_DLL_NAME "xvidcore.dll"
-
 CXvidDecoder::CXvidDecoder(LPUNKNOWN punk, HRESULT *phr) :
     CVideoTransformFilter(NAME("CXvidDecoder"), punk, CLSID_XVID), m_hdll (NULL)
 {
@@ -322,20 +317,39 @@ CXvidDecoder::CXvidDecoder(LPUNKNOWN punk, HRESULT *phr) :
     xvid_decore_func = NULL; // Hmm, some strange errors appearing if I try to initialize...
     xvid_global_func = NULL; // ...this in constructor's init-list. So, they assigned here.
 
+	m_tray_icon = 0;
+	m_startClock = clock();
+	interlaced = 0;
+
 #if defined(XVID_USE_MFT)
 	InitializeCriticalSection(&m_mft_lock);
 	m_pInputType = NULL;
 	m_pOutputType = NULL;
+	m_pOutputTypeBPP = 32;
 	m_rtFrame = 0;
 	m_duration = 0;
 	m_discont = 0;
 	m_frameRate.Denominator = 1;
 	m_frameRate.Numerator = 1;
+	m_thread_handle = NULL;
 #endif
 
     LoadRegistryInfo();
 
     *phr = OpenLib();
+	
+	{
+		TCHAR lpFilename[MAX_PATH];
+		int sLen = GetModuleFileName(NULL, lpFilename, MAX_PATH);
+#ifdef _UNICODE
+		if ((sLen >= 11) && (_wcsnicmp(&(lpFilename[sLen - 11]), TEXT("dllhost.exe"), 11) == 0)) {
+#else
+		if ((sLen >= 11) && (_strnicmp(&(lpFilename[sLen - 11]), TEXT("dllhost.exe"), 11) == 0)) {
+#endif
+			if (m_tray_icon == 0) m_tray_icon = -1; // create no tray icon upon thumbnail generation
+		}
+	}
+
 }
 
 HRESULT CXvidDecoder::OpenLib()
@@ -497,23 +511,31 @@ CXvidDecoder::~CXvidDecoder()
 {
     DPRINTF("Destructor");
 
-#ifdef XVID_USE_TRAYICON
-	if (Tray_Icon) { /* Destroy tray icon */
-		NOTIFYICONDATA nid;
-		ZeroMemory(&nid,sizeof(NOTIFYICONDATA));
-
-		nid.cbSize = NOTIFYICONDATA_V1_SIZE;
-		nid.hWnd = MSG_hwnd;
-		nid.uID = 1456;
-	
-		Shell_NotifyIcon(NIM_DELETE, &nid);
-		Tray_Icon = 0;
+	if ((MSG_hwnd != NULL) && (m_tray_icon == 1)) { /* Destroy tray icon */
+		SendMessage(MSG_hwnd, WM_CLOSE, 0, 0);
 	}
-#endif
-
+	
 	/* Close xvidcore library */
 	CloseLib();
 
+	clock_t endClock = clock();
+	if (((endClock - m_startClock) / CLOCKS_PER_SEC) > 3) {
+		SaveRegistryInfo((endClock - m_startClock) / (CLOCKS_PER_SEC / 10));
+	}
+
+	if ((MSG_hwnd != 0) && (m_tray_icon == 1)) { /* Final clean-up */
+	  MSG_hwnd = 0;
+	  Sleep(200);
+	  m_tray_icon = 0;
+#if defined(XVID_USE_MFT)
+	  if (m_thread_handle) {
+		  TerminateThread(m_thread_handle, 0);
+		  CloseHandle(m_thread_handle);
+		  m_thread_handle = NULL;
+	  }
+#endif
+	}
+
 #if defined(XVID_USE_MFT)
 	DeleteCriticalSection(&m_mft_lock);
 #endif
@@ -707,6 +729,15 @@ HRESULT CXvidDecoder::GetMediaType(int iPosition, CMediaType *mtOut)
 			vih->dwPictAspectRatioY = abs(m_create.height);
 			forced_ar = false;
 		} 
+		if (interlaced) {
+			vih->dwInterlaceFlags = AMINTERLACE_IsInterlaced;
+			if (interlaced > 1) {
+				vih->dwInterlaceFlags |= AMINTERLACE_Field1First;
+			}
+		}
+		else {
+			vih->dwInterlaceFlags = 0;
+		}
 	} else {
 
 		VIDEOINFOHEADER * vih = (VIDEOINFOHEADER *) mtOut->ReallocFormatBuffer(sizeof(VIDEOINFOHEADER));
@@ -814,16 +845,18 @@ if ( USE_RG565 )
 
 
 /* (internal function) change colorspace */
-#define CALC_BI_STRIDE(width,bitcount)  ((((width * bitcount) + 31) & ~31) >> 3)
+#define CALC_BI_STRIDE(width, bitcount)  ((((width * bitcount) + 31) & ~31) >> 3)
 
-HRESULT CXvidDecoder::ChangeColorspace(GUID subtype, GUID formattype, void * format, int noflip)
+HRESULT CXvidDecoder::ChangeColorspace(GUID subtype, GUID formattype, void *format, int *bitdepth, int noflip)
 {
 	DWORD biWidth;
+	*bitdepth = 32;
 
 	if (formattype == FORMAT_VideoInfo)
 	{
 		VIDEOINFOHEADER * vih = (VIDEOINFOHEADER * )format;
 		biWidth = vih->bmiHeader.biWidth;
+		*bitdepth = vih->bmiHeader.biBitCount;
 		out_stride = CALC_BI_STRIDE(vih->bmiHeader.biWidth, vih->bmiHeader.biBitCount);
 		rgb_flip = (vih->bmiHeader.biHeight < 0 ? 0 : XVID_CSP_VFLIP);
 	}
@@ -831,6 +864,7 @@ HRESULT CXvidDecoder::ChangeColorspace(GUID subtype, GUID formattype, void * for
 	{
 		VIDEOINFOHEADER2 * vih2 = (VIDEOINFOHEADER2 * )format;
 		biWidth = vih2->bmiHeader.biWidth;
+		*bitdepth = vih2->bmiHeader.biBitCount;
 		out_stride = CALC_BI_STRIDE(vih2->bmiHeader.biWidth, vih2->bmiHeader.biBitCount);
 		rgb_flip = (vih2->bmiHeader.biHeight < 0 ? 0 : XVID_CSP_VFLIP);
 	}
@@ -910,11 +944,12 @@ HRESULT CXvidDecoder::ChangeColorspace(GUID subtype, GUID formattype, void * for
 
 HRESULT CXvidDecoder::SetMediaType(PIN_DIRECTION direction, const CMediaType *pmt)
 {
+	int bitdepth;
 	DPRINTF("SetMediaType");
 	
 	if (direction == PINDIR_OUTPUT)
 	{
-		return ChangeColorspace(*pmt->Subtype(), *pmt->FormatType(), pmt->Format(), 0);
+		return ChangeColorspace(*pmt->Subtype(), *pmt->FormatType(), pmt->Format(), &bitdepth, 0);
 	}
 	
 	return S_OK;
@@ -936,8 +971,7 @@ HRESULT CXvidDecoder::CompleteConnect(PIN_DIRECTION direction, IPin *pReceivePin
 {
 	DPRINTF("CompleteConnect");
 
-#ifdef XVID_USE_TRAYICON
-	if ((direction == PINDIR_OUTPUT) && (Tray_Icon == 0)) 
+	if ((direction == PINDIR_OUTPUT) && (MSG_hwnd == 0) && (m_tray_icon == 0) && (g_config.bTrayIcon != 0))
 	{
 		WNDCLASSEX wc; 
 
@@ -962,20 +996,21 @@ HRESULT CXvidDecoder::CompleteConnect(PIN_DIRECTION direction, IPin *pReceivePin
 		NOTIFYICONDATA nid;    
 		ZeroMemory(&nid,sizeof(NOTIFYICONDATA));
 
-		nid.cbSize = NOTIFYICONDATA_V1_SIZE;
+		nid.cbSize = sizeof(NOTIFYICONDATA);
 		nid.hWnd = MSG_hwnd;  
 		nid.uID = 1456;  
+		nid.uVersion = NOTIFYICON_VERSION;
 		nid.uCallbackMessage = WM_ICONMESSAGE;  
 		nid.hIcon = LoadIcon(g_xvid_hInst, MAKEINTRESOURCE(IDI_ICON));  
 		strcpy_s(nid.szTip, 19, "Xvid Video Decoder");  
-		nid.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
+		nid.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP | NIF_SHOWTIP;
 	
 		Shell_NotifyIcon(NIM_ADD, &nid); 
+		Shell_NotifyIcon(NIM_SETVERSION, &nid);
 
 		DestroyIcon(nid.hIcon);
-		Tray_Icon = 1;
+		m_tray_icon = 1;
 	}
-#endif
 
 	return S_OK;
 }
@@ -1049,9 +1084,10 @@ HRESULT CXvidDecoder::Transform(IMediaSample *pIn, IMediaSample *pOut)
 	pOut->GetMediaType(&mtOut);
 	if (mtOut != NULL)
 	{
+		int bitdepth;
 		HRESULT result;
 
-		result = ChangeColorspace(mtOut->subtype, mtOut->formattype, mtOut->pbFormat, 0);
+		result = ChangeColorspace(mtOut->subtype, mtOut->formattype, mtOut->pbFormat, &bitdepth, 0);
 		DeleteMediaType(mtOut);
 
 		if (result != S_OK)
@@ -1115,30 +1151,49 @@ repeat :
 		{
             DPRINTF("*** XVID_DEC_DECODE");
 			return S_FALSE;
-		} else 
-			if (g_config.aspect_ratio == 0 || g_config.aspect_ratio == 1 && forced_ar == false) {
+		} else if ((g_config.aspect_ratio == 0 || g_config.aspect_ratio == 1 && forced_ar == false) || !!(interlaced)){
         
-      if (stats.type != XVID_TYPE_NOTHING) {  /* dont attempt to set vmr aspect ratio if no frame was returned by decoder */
-			// inspired by minolta! works for VMR 7 + 9
-			IMediaSample2 *pOut2 = NULL;
-			AM_SAMPLE2_PROPERTIES outProp2;
-			if (SUCCEEDED(pOut->QueryInterface(IID_IMediaSample2, (void **)&pOut2)) &&
-				SUCCEEDED(pOut2->GetProperties(FIELD_OFFSET(AM_SAMPLE2_PROPERTIES, tStart), (PBYTE)&outProp2)))
-			{
-				CMediaType mtOut2 = m_pOutput->CurrentMediaType();
-				VIDEOINFOHEADER2* vihOut2 = (VIDEOINFOHEADER2*)mtOut2.Format();
-
-				if (*mtOut2.FormatType() == FORMAT_VideoInfo2 && 
-					vihOut2->dwPictAspectRatioX != ar_x && vihOut2->dwPictAspectRatioY != ar_y)
+			if (stats.type != XVID_TYPE_NOTHING) {  /* dont attempt to set vmr aspect ratio if no frame was returned by decoder */
+				// inspired by minolta! works for VMR 7 + 9
+				IMediaSample2 *pOut2 = NULL;
+				AM_SAMPLE2_PROPERTIES outProp2;
+				if (SUCCEEDED(pOut->QueryInterface(IID_IMediaSample2, (void **)&pOut2)) &&
+					SUCCEEDED(pOut2->GetProperties(FIELD_OFFSET(AM_SAMPLE2_PROPERTIES, tStart), (PBYTE)&outProp2)))
 				{
-					vihOut2->dwPictAspectRatioX = ar_x;
-					vihOut2->dwPictAspectRatioY = ar_y;
-					pOut2->SetMediaType(&mtOut2);
-					m_pOutput->SetMediaType(&mtOut2);
+					CMediaType mtOut2 = m_pOutput->CurrentMediaType();
+					VIDEOINFOHEADER2* vihOut2 = (VIDEOINFOHEADER2*)mtOut2.Format();
+
+					if (*mtOut2.FormatType() == FORMAT_VideoInfo2) 
+					{
+						int need_format_change = 0;
+
+						if (vihOut2->dwPictAspectRatioX != ar_x || vihOut2->dwPictAspectRatioY != ar_y)
+						{
+							vihOut2->dwPictAspectRatioX = ar_x;
+							vihOut2->dwPictAspectRatioY = ar_y;
+							need_format_change = 1;
+						}
+						if ((interlaced) && !(vihOut2->dwInterlaceFlags & AMINTERLACE_IsInterlaced)) 
+						{
+							vihOut2->dwInterlaceFlags = AMINTERLACE_IsInterlaced;
+							if (interlaced > 2) {
+								vihOut2->dwInterlaceFlags |= AMINTERLACE_Field1First;
+							}
+							need_format_change = 1;
+						}
+						else if (!interlaced && !!(vihOut2->dwInterlaceFlags & AMINTERLACE_IsInterlaced)){
+							vihOut2->dwInterlaceFlags = 0;
+							need_format_change = 1;
+						}
+
+						if (need_format_change) {
+							pOut2->SetMediaType(&mtOut2);
+							m_pOutput->SetMediaType(&mtOut2);
+						}
+					}
+					pOut2->Release();
 				}
-				pOut2->Release();
 			}
-      }
 		}
 	}
 	else
@@ -1198,6 +1253,13 @@ repeat :
 			ar_y = par_y * stats.data.vol.height;
 		}
 
+		if (!!(stats.data.vol.general & XVID_VOL_INTERLACING)) {
+			interlaced = (stats.data.vop.general & XVID_VOP_TOPFIELDFIRST) ? 2 : 1;
+		}
+		else if (interlaced > 0) {
+			interlaced = 0;
+		}
+
 		m_frame.bitstream = (BYTE*)m_frame.bitstream + length;
 		m_frame.length -= length;
 		goto repeat;
@@ -1321,7 +1383,7 @@ HRESULT CXvidDecoder::MFTGetOutputStreamInfo(DWORD dwOutputStreamID, MFT_OUTPUT_
 		pStreamInfo->cbAlignment = 0;
 	}
 	else {
-		pStreamInfo->cbSize = m_create.width * abs(m_create.height) * 4; // XXX
+		pStreamInfo->cbSize = (m_create.width * abs(m_create.height) * m_pOutputTypeBPP) >> 3;
 		pStreamInfo->cbAlignment = 1;
 	}
 
@@ -1367,7 +1429,7 @@ HRESULT CXvidDecoder::MFTGetInputAvailableType(DWORD dwInputStreamID, DWORD dwTy
 		return MF_E_INVALIDSTREAMNUMBER;
 
 	DWORD i = 0;
-	GUID *bs_guid_table[8];
+	GUID *bs_guid_table[32];
 
 	bs_guid_table[i++] = (GUID *)&CLSID_XVID;
 	bs_guid_table[i++] = (GUID *)&CLSID_XVID_UC;
@@ -1458,49 +1520,56 @@ if ( USE_YUY2 )
 		bitdepth = 4;
 		break;
 }
-	case 1 :
+	case 1:
+if ( USE_YVYU )
+{
+	    csp = MFVideoFormat_YVYU;
+	    bitdepth = 4;
+	    break;
+}
+	case 2 :
 if ( USE_UYVY )
 {
 		csp = MFVideoFormat_UYVY;
 		bitdepth = 4;
 		break;
 }
-	case 2	:
+	case 3	:
 		if ( USE_IYUV )
 {
 		csp = MFVideoFormat_IYUV;
 		bitdepth = 3;
 		break;
 }
-	case 3	:
+	case 4	:
 if ( USE_YV12 )
 {
 		csp = MFVideoFormat_YV12;
 		bitdepth = 3;
 		break;
 }
-	case 4 :
+	case 5 :
 if ( USE_RGB32 )
 {
 		csp = MFVideoFormat_RGB32;
 		bitdepth = 8;
 		break;
 }
-	case 5 :
+	case 6 :
 if ( USE_RGB24 )
 {
 		csp = MFVideoFormat_RGB24;
 		bitdepth = 6;
 		break;
 }
-	case 6 :
+	case 7 :
 if ( USE_RG555 )
 {
 		csp = MFVideoFormat_RGB555;
 		bitdepth = 4;
 		break;
 }
-	case 7 :
+	case 8 :
 if ( USE_RG565 )
 {
 		csp = MFVideoFormat_RGB565;
@@ -1538,11 +1607,11 @@ if ( USE_RG565 )
 	}
 	
 	if (SUCCEEDED(hr)) {
-		hr = pOutputType->SetUINT32(MF_MT_SAMPLE_SIZE, (m_create.height * m_create.width * bitdepth)>>1);
+		hr = pOutputType->SetUINT32(MF_MT_SAMPLE_SIZE, (abs(m_create.height) * m_create.width * bitdepth) >> 1);
 	}
 	
 	if (SUCCEEDED(hr)) {
-		hr = MFSetAttributeSize(pOutputType, MF_MT_FRAME_SIZE, m_create.width, m_create.height);
+		hr = MFSetAttributeSize(pOutputType, MF_MT_FRAME_SIZE, m_create.width, abs(m_create.height));
 	}
 	
 	if (SUCCEEDED(hr)) {
@@ -1550,7 +1619,15 @@ if ( USE_RG565 )
 	}
 	
 	if (SUCCEEDED(hr)) {
-		hr = pOutputType->SetUINT32(MF_MT_INTERLACE_MODE, MFVideoInterlace_Progressive);
+		if (interlaced > 1) {
+			hr = pOutputType->SetUINT32(MF_MT_INTERLACE_MODE, MFVideoInterlace_FieldInterleavedUpperFirst);
+		}
+		else if (interlaced) {
+			hr = pOutputType->SetUINT32(MF_MT_INTERLACE_MODE, MFVideoInterlace_FieldInterleavedLowerFirst);
+		}
+		else {
+			hr = pOutputType->SetUINT32(MF_MT_INTERLACE_MODE, MFVideoInterlace_Progressive);
+		}
 	}
 	
 	if (SUCCEEDED(hr)) {
@@ -1590,7 +1667,7 @@ HRESULT CXvidDecoder::MFTSetInputType(DWORD dwInputStreamID, IMFMediaType *pType
 		hr = MF_E_TRANSFORM_CANNOT_CHANGE_MEDIATYPE_WHILE_PROCESSING;
 
 	if (SUCCEEDED(hr)) { 
-        if (pType) { // /* Check the type */
+        if (pType) { /* Check the type */
             hr = OnCheckInputType(pType);
         }
 	}
@@ -1605,6 +1682,54 @@ HRESULT CXvidDecoder::MFTSetInputType(DWORD dwInputStreamID, IMFMediaType *pType
 	return hr;
 }
 
+DWORD WINAPI CreateTrayIcon(LPVOID lpParameter)
+{
+	WNDCLASSEX wc;
+
+	wc.cbSize = sizeof(WNDCLASSEX);
+	wc.lpfnWndProc = msg_proc;
+	wc.style = CS_HREDRAW | CS_VREDRAW;
+	wc.cbWndExtra = 0;
+	wc.cbClsExtra = 0;
+	wc.hInstance = (HINSTANCE)g_xvid_hInst;
+	wc.hbrBackground = (HBRUSH)GetStockObject(NULL_BRUSH);
+	wc.lpszMenuName = NULL;
+	wc.lpszClassName = "XVID_MSG_WINDOW";
+	wc.hIcon = NULL;
+	wc.hIconSm = NULL;
+	wc.hCursor = NULL;
+	RegisterClassEx(&wc);
+
+	MSG_hwnd = CreateWindowEx(0, "XVID_MSG_WINDOW", NULL, 0, CW_USEDEFAULT,
+		CW_USEDEFAULT, 0, 0, HWND_MESSAGE, NULL, (HINSTANCE)g_xvid_hInst, NULL);
+
+	/* display the tray icon */
+	NOTIFYICONDATA nid;
+	ZeroMemory(&nid, sizeof(NOTIFYICONDATA));
+
+	nid.cbSize = sizeof(NOTIFYICONDATA);
+	nid.hWnd = MSG_hwnd;
+	nid.uID = 1456;
+	nid.uVersion = NOTIFYICON_VERSION;
+	nid.uCallbackMessage = WM_ICONMESSAGE;
+	nid.hIcon = LoadIcon(g_xvid_hInst, MAKEINTRESOURCE(IDI_ICON));
+	strcpy_s(nid.szTip, 19, "Xvid Video Decoder");
+	nid.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP | NIF_SHOWTIP;
+
+	Shell_NotifyIcon(NIM_ADD, &nid);
+	Shell_NotifyIcon(NIM_SETVERSION, &nid);
+
+	DestroyIcon(nid.hIcon);
+	
+	MSG msg;
+	while (MSG_hwnd && GetMessage(&msg, MSG_hwnd, 0, 0)) {
+		TranslateMessage(&msg);
+		DispatchMessage(&msg);
+	}
+	
+	return 0;
+}
+
 HRESULT CXvidDecoder::MFTSetOutputType(DWORD dwOutputStreamID, IMFMediaType *pType, DWORD dwFlags)
 {
 	DPRINTF("(MFT)SetOutputType");
@@ -1626,13 +1751,14 @@ HRESULT CXvidDecoder::MFTSetOutputType(DWORD dwOutputStreamID, IMFMediaType *pTy
 	if (HasPendingOutput())
 		hr = MF_E_TRANSFORM_CANNOT_CHANGE_MEDIATYPE_WHILE_PROCESSING;
 
+	int bitdepth;
 	if (SUCCEEDED(hr)) { 
 		if (pType) { /* Check the type */
 			AM_MEDIA_TYPE *am;
 			hr = MFCreateAMMediaTypeFromMFMediaType(pType, GUID_NULL, &am);
 			
 			if (SUCCEEDED(hr)) {
-				if (FAILED(ChangeColorspace(am->subtype, am->formattype, am->pbFormat, 1))) {
+				if (FAILED(ChangeColorspace(am->subtype, am->formattype, am->pbFormat, &bitdepth, 1))) {
 					DPRINTF("(MFT)InternalCheckOutputType (MF_E_INVALIDTYPE)");
 					return MF_E_INVALIDTYPE;
 				}
@@ -1645,51 +1771,20 @@ HRESULT CXvidDecoder::MFTSetOutputType(DWORD dwOutputStreamID, IMFMediaType *pTy
 	
 	if (SUCCEEDED(hr)) {
 		if (bReallySet) { /* Set the type if needed */
-			hr = OnSetOutputType(pType);
+			hr = OnSetOutputType(pType, bitdepth);
 		}
 	}
-#ifdef XVID_USE_TRAYICON
-	if (SUCCEEDED(hr) && Tray_Icon == 0) /* Create message passing window */
+
+	if (SUCCEEDED(hr) && (MSG_hwnd == 0) && (m_tray_icon == 0) && (g_config.bTrayIcon != 0))  
 	{
-		WNDCLASSEX wc; 
+		m_thread_handle = CreateThread(NULL, 0, CreateTrayIcon, NULL, 0, NULL);  /* Create message passing window */
 
-		wc.cbSize = sizeof(WNDCLASSEX);
-		wc.lpfnWndProc = msg_proc;
-		wc.style = CS_HREDRAW | CS_VREDRAW;
-		wc.cbWndExtra = 0;
-		wc.cbClsExtra = 0;
-		wc.hInstance = (HINSTANCE) g_xvid_hInst;
-		wc.hbrBackground = (HBRUSH) GetStockObject(NULL_BRUSH);
-		wc.lpszMenuName = NULL;
-		wc.lpszClassName = "XVID_MSG_WINDOW";
-		wc.hIcon = NULL;
-		wc.hIconSm = NULL;
-		wc.hCursor = NULL;
-		RegisterClassEx(&wc);
-
-		MSG_hwnd = CreateWindowEx(0, "XVID_MSG_WINDOW", NULL, 0, CW_USEDEFAULT, 
-                                  CW_USEDEFAULT, 0, 0, HWND_MESSAGE, NULL, (HINSTANCE) g_xvid_hInst, NULL);
-
-		/* display the tray icon */
-		NOTIFYICONDATA nid;    
-		ZeroMemory(&nid,sizeof(NOTIFYICONDATA));
-
-		nid.cbSize = NOTIFYICONDATA_V1_SIZE;
-		nid.hWnd = MSG_hwnd;  
-		nid.uID = 1456;  
-		nid.uCallbackMessage = WM_ICONMESSAGE;  
-		nid.hIcon = LoadIcon(g_xvid_hInst, MAKEINTRESOURCE(IDI_ICON));  
-		strcpy_s(nid.szTip, 19, "Xvid Video Decoder");  
-		nid.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
-		
-		Shell_NotifyIcon(NIM_ADD, &nid); 
-
-		DestroyIcon(nid.hIcon);
-		Tray_Icon = 1;
+		if (m_thread_handle)
+		    m_tray_icon = 1;
 	}
-#endif
 
 	LeaveCriticalSection(&m_mft_lock);
+
 	return hr;
 }
 
@@ -1757,7 +1852,7 @@ HRESULT CXvidDecoder::MFTGetInputStatus(DWORD dwInputStreamID, DWORD *pdwFlags)
 	
 	EnterCriticalSection(&m_mft_lock);
 	
-	/* If there's pending output sampels we don't accept new
+	/* If there's pending output samples we don't accept new
 	   input data until ProcessOutput() or Flush() was called */
 	if (!HasPendingOutput()) {
 		*pdwFlags = MFT_INPUT_STATUS_ACCEPT_DATA;
@@ -1910,6 +2005,8 @@ HRESULT CXvidDecoder::MFTProcessInput(DWORD dwInputStreamID, IMFSample *pSample,
 	HRESULT hr = S_OK;
 	IMFMediaBuffer *pBuffer;
 
+	int need_format_change = 0;
+
 	if (SUCCEEDED(hr)) {
 		hr = pSample->ConvertToContiguousBuffer(&pBuffer);
 	}
@@ -1993,8 +2090,21 @@ repeat :
 				par_y = PARS[stats.data.vol.par-1][1];
 			}
 
-			ar_x = par_x * stats.data.vol.width;
-			ar_y = par_y * stats.data.vol.height;
+			ar_x = par_x;
+			ar_y = par_y; 
+
+			need_format_change = 1;
+		}
+
+		if (!!(stats.data.vol.general & XVID_VOL_INTERLACING)) {
+			interlaced = (stats.data.vop.general & XVID_VOP_TOPFIELDFIRST) ? 2 : 1;
+			need_format_change = 1;
+		}
+		else {
+			if (interlaced > 0) {
+				interlaced = 0;
+				need_format_change = 1;
+			}
 		}
 
 		m_frame.bitstream = (BYTE*)m_frame.bitstream + length;
@@ -2023,6 +2133,33 @@ END_LOOP:
 		}
 	}
 	
+	if (need_format_change) {
+		IMFMediaType *pOutputType = NULL;
+		hr = MFTGetOutputCurrentType(0, &pOutputType);
+		
+		if (SUCCEEDED(hr)) {
+	    	if (interlaced > 1) {
+		        hr = pOutputType->SetUINT32(MF_MT_INTERLACE_MODE, MFVideoInterlace_FieldInterleavedUpperFirst);
+		    }
+		    else if (interlaced) {
+		        hr = pOutputType->SetUINT32(MF_MT_INTERLACE_MODE, MFVideoInterlace_FieldInterleavedLowerFirst);
+		    }
+		    else {
+		        hr = pOutputType->SetUINT32(MF_MT_INTERLACE_MODE, MFVideoInterlace_Progressive);
+		    }
+		}
+		
+		if (SUCCEEDED(hr)) {
+		    hr = MFSetAttributeRatio(pOutputType, MF_MT_PIXEL_ASPECT_RATIO, ar_x, ar_y);
+		}
+
+		if (SUCCEEDED(hr)) {
+			MFTSetOutputType(0, pOutputType, 0);
+		}
+
+		if (pOutputType) pOutputType->Release();
+	}
+
 	LeaveCriticalSection(&m_mft_lock);
 
 	return hr;
@@ -2056,19 +2193,28 @@ HRESULT CXvidDecoder::MFTProcessOutput(DWORD dwFlags, DWORD cOutputBufferCount,
 	
 	BYTE *Dst = NULL;
 	DWORD buffer_size;
-	
+	LONG stride = (m_pOutputTypeBPP < 15) ? CALC_BI_STRIDE(m_create.width, 8) : CALC_BI_STRIDE(m_create.width, m_pOutputTypeBPP);
 	IMFMediaBuffer *pOutput = NULL;
-	
+	IMF2DBuffer *pOutput2D = NULL;
+
 	if (SUCCEEDED(hr)) {
 		hr = pOutputSamples[0].pSample->GetBufferByIndex(0, &pOutput); /* Get output buffer */
 	}
-	
+
 	if (SUCCEEDED(hr)) {
 		hr = pOutput->GetMaxLength(&buffer_size);
 	}
-	
-	if (SUCCEEDED(hr))
+
+	if (SUCCEEDED(hr)) {
+		hr = pOutput->QueryInterface(IID_IMF2DBuffer, (void **)&pOutput2D);
+	}
+
+	if (SUCCEEDED(hr)) {
+		hr = pOutput2D->Lock2D(&Dst, &stride);
+	}
+	else {
 		hr = pOutput->Lock(&Dst, NULL, NULL);
+	}
 
 	if (SUCCEEDED(hr)) {
 		xvid_gbl_convert_t convert;
@@ -2086,16 +2232,15 @@ HRESULT CXvidDecoder::MFTProcessOutput(DWORD dwFlags, DWORD cOutputBufferCount,
 
 		convert.output.csp = m_frame.output.csp;
 		convert.output.plane[0] = Dst;
-		convert.output.stride[0] = out_stride;
+		convert.output.stride[0] = stride;
 
 		convert.width = m_create.width;
 		convert.height = m_create.height;
-		convert.interlacing = 0;
+		convert.interlacing = (interlaced > 0) ? 1 : 0;
 		
 		if (m_frame.output.plane[1] != NULL && Dst != NULL && xvid_global_func != NULL) 
 			if (xvid_global_func(0, XVID_GBL_CONVERT, &convert, NULL) < 0) /* CSP convert into output buffer */
 				hr = E_FAIL;
-
 		m_frame.output.plane[1] = NULL;
 	}
 
@@ -2120,14 +2265,20 @@ HRESULT CXvidDecoder::MFTProcessOutput(DWORD dwFlags, DWORD cOutputBufferCount,
 		}
 		
 		if (SUCCEEDED(hr))
-			hr = pOutput->SetCurrentLength(m_create.width * abs(m_create.height) * 4); // XXX
+			hr = pOutput->SetCurrentLength((m_create.width * abs(m_create.height) * m_pOutputTypeBPP) >> 3);
 	}
 	
-	if (pOutput) { 
-		pOutput->Unlock(); 
-		pOutput->Release(); 
+	if (pOutput2D) { 
+		pOutput2D->Unlock2D();
+		pOutput2D->Release();
+		if (pOutput)
+		    pOutput->Release(); 
 	}
-	
+	else if (pOutput) {
+		pOutput->Unlock();
+		pOutput->Release();
+	}
+
 	LeaveCriticalSection(&m_mft_lock);
 
 	return hr;
@@ -2241,7 +2392,7 @@ HRESULT CXvidDecoder::OnCheckInputType(IMFMediaType *pmt)
 		CloseLib();
 		hr = MF_E_INVALIDTYPE;
 	}
-	
+
 	/* haali media splitter reports VOL information in the format header */
 	if (SUCCEEDED(hr))
 	{
@@ -2311,13 +2462,14 @@ HRESULT CXvidDecoder::OnSetInputType(IMFMediaType *pmt)
 	return hr;
 }
 
-HRESULT CXvidDecoder::OnSetOutputType(IMFMediaType *pmt)
+HRESULT CXvidDecoder::OnSetOutputType(IMFMediaType *pmt, int bitdepth)
 {
 	if (m_pOutputType) m_pOutputType->Release();
 	
 	m_pOutputType = pmt;
 	m_pOutputType->AddRef();
-	
+	m_pOutputTypeBPP = bitdepth;
+
 	return S_OK;
 }
 
