@@ -3,7 +3,7 @@
  *  XVID MPEG-4 VIDEO CODEC
  *  - XviD Decoder part of the DShow Filter  -
  *
- *  Copyright(C) 2002-2010 Peter Ross <pross@xvid.org>
+ *  Copyright(C) 2002-2012 Peter Ross <pross@xvid.org>
  *
  *  This program is free software ; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,17 +19,19 @@
  *  along with this program ; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * $Id: CXvidDecoder.h,v 1.9.2.2 2011/03/17 15:13:25 Isibaar Exp $
+ * $Id: CXvidDecoder.h 2131 2016-01-08 17:44:53Z Isibaar $
  *
  ****************************************************************************/
 
 #ifndef _FILTER_H_
 #define _FILTER_H_
 
+#include <time.h>
 #include <xvid.h>
 #include "IXvidDecoder.h"
 
 #define XVID_NAME_L		L"Xvid MPEG-4 Video Decoder"
+#define XVID_NAME_MFT_L	L"Xvid MPEG-4 Video Decoder MFT"
 
 /* --- fourcc --- */
 
@@ -153,7 +155,7 @@ public :
 
 private :
 
-	HRESULT ChangeColorspace(GUID subtype, GUID formattype, void * format, int noflip);
+	HRESULT ChangeColorspace(GUID subtype, GUID formattype, void * format, int *bitdepth, int noflip);
 	HRESULT OpenLib();
 	void CloseLib();
 
@@ -164,11 +166,15 @@ private :
 	int (*xvid_global_func)(void *handle, int opt, void *param1, void *param2);
 	int (*xvid_decore_func)(void *handle, int opt, void *param1, void *param2);
 	int ar_x, ar_y;
+	int interlaced;
 	bool forced_ar;
 
 	int rgb_flip;
 	int out_stride;
 
+	clock_t m_startClock;
+	int m_tray_icon;
+
 	/* mft stuff */
 #if defined(XVID_USE_MFT)
 	BOOL HasPendingOutput() const { return m_frame.output.plane[1] != NULL; }
@@ -176,10 +182,11 @@ private :
 	HRESULT OnSetInputType(IMFMediaType *pmt);
 	HRESULT OnCheckInputType(IMFMediaType *pmt);
 
-	HRESULT OnSetOutputType(IMFMediaType *pmt);
+	HRESULT OnSetOutputType(IMFMediaType *pmt, int bitdepth);
 
 	IMFMediaType *m_pInputType;
 	IMFMediaType *m_pOutputType;
+	int m_pOutputTypeBPP;
 
 	CRITICAL_SECTION m_mft_lock;
 	REFERENCE_TIME m_timestamp;
@@ -191,15 +198,11 @@ private :
 	REFERENCE_TIME m_rtFrame;
 	MFRatio m_frameRate;
 	UINT64 m_duration;
-#endif
 
-#ifdef XVID_USE_TRAYICON
-	HWND MSG_hwnd; /* message handler window */
+	HANDLE m_thread_handle;
+#endif
 };
 #define WM_ICONMESSAGE (WM_USER + 1)
-#else
-};
-#endif
 
 static const int PARS[][2] = {
 	{1, 1},
