@@ -20,7 +20,7 @@
  *  along with this program ; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * $Id: bitstream.c 1985 2011-05-18 09:02:35Z Isibaar $
+ * $Id: bitstream.c 2180 2019-11-12 14:48:35Z Isibaar $
  *
  ****************************************************************************/
 
@@ -153,6 +153,8 @@ read_video_packet_header(Bitstream *bs,
 		READ_MARKER();
 		if (dec->time_inc_bits)
 			time_increment = (BitstreamGetBits(bs, dec->time_inc_bits));	/* vop_time_increment */
+		else
+			time_increment = 0;
 		READ_MARKER();
 		DPRINTF(XVID_DEBUG_HEADER,"time %i:%i\n", time_base, time_increment);
 
@@ -398,7 +400,6 @@ BitstreamReadHeaders(Bitstream * bs,
 	uint32_t start_code;
 	uint32_t time_incr = 0;
 	int32_t time_increment = 0;
-	int resize = 0;
 
 	while ((BitstreamPos(bs) >> 3) + 4 <= bs->length) {
 
@@ -466,6 +467,7 @@ BitstreamReadHeaders(Bitstream * bs,
 			BitstreamSkip(bs, 32);	/* video_object_start_code */
 
 		} else if ((start_code & ~VIDOBJLAY_START_CODE_MASK) == VIDOBJLAY_START_CODE) {
+			uint32_t width = 0, height = 0;
 
 			DPRINTF(XVID_DEBUG_STARTCODE, "<video_object_layer>\n");
 			DPRINTF(XVID_DEBUG_HEADER, "vol id %i\n", start_code & VIDOBJLAY_START_CODE_MASK);
@@ -566,8 +568,6 @@ BitstreamReadHeaders(Bitstream * bs,
 			if (dec->shape != VIDOBJLAY_SHAPE_BINARY_ONLY) {
 
 				if (dec->shape == VIDOBJLAY_SHAPE_RECTANGULAR) {
-					uint32_t width, height;
-
 					READ_MARKER();
 					width = BitstreamGetBits(bs, 13);	/* video_object_layer_width */
 					READ_MARKER();
@@ -576,18 +576,6 @@ BitstreamReadHeaders(Bitstream * bs,
 
 					DPRINTF(XVID_DEBUG_HEADER, "width %i\n", width);
 					DPRINTF(XVID_DEBUG_HEADER, "height %i\n", height);
-
-					if (dec->width != width || dec->height != height)
-					{
-						if (dec->fixed_dimensions)
-						{
-							DPRINTF(XVID_DEBUG_ERROR, "decoder width/height does not match bitstream\n");
-							return -1;
-						}
-						resize = 1;
-						dec->width = width;
-						dec->height = height;
-					}
 				}
 
 				dec->interlacing = BitstreamGetBit(bs);
@@ -764,7 +752,19 @@ BitstreamReadHeaders(Bitstream * bs,
 
 			}
 
-			return (resize ? -3 : -2 );	/* VOL */
+			if (((width > 0) && (height > 0)) && (dec->width != width || dec->height != height))
+			{
+				if (dec->fixed_dimensions)
+				{
+					DPRINTF(XVID_DEBUG_ERROR, "decoder width/height does not match bitstream\n");
+					return -1;
+				}
+				dec->width = width;
+				dec->height = height;
+				return -3;
+			}
+
+			return -2;	/* VOL */
 
 		} else if (start_code == GRPOFVOP_START_CODE) {
 
@@ -860,8 +860,9 @@ BitstreamReadHeaders(Bitstream * bs,
 				dec->shape == VIDOBJLAY_SHAPE_RECTANGULAR &&
 				(coding_type == P_VOP || coding_type == I_VOP)) {
 
-				if (BitstreamGetBit(bs));
+				if (BitstreamGetBit(bs)) {
 					DPRINTF(XVID_DEBUG_ERROR, "RRV not supported (anymore)\n");
+                                }
 			}
 
 			if (dec->shape != VIDOBJLAY_SHAPE_RECTANGULAR) {
@@ -919,7 +920,7 @@ BitstreamReadHeaders(Bitstream * bs,
 
 				int i;
 
-				for (i = 0 ; i < dec->sprite_warping_points; i++)
+				for (i = 0 ; i < MIN(4, dec->sprite_warping_points); i++)
 				{
 					int length;
 					int x = 0, y = 0;
@@ -981,7 +982,7 @@ BitstreamReadHeaders(Bitstream * bs,
 
 		} else if (start_code == USERDATA_START_CODE) {
 			char tmp[256];
-		    int i, version, build;
+                        int i, version = 0, build = 0;
 			char packed;
 
 			BitstreamSkip(bs, 32);	/* user_data_start_code */
@@ -989,7 +990,7 @@ BitstreamReadHeaders(Bitstream * bs,
 			memset(tmp, 0, 256);
 			tmp[0] = BitstreamShowBits(bs, 8);
 
-			for(i = 1; i < 256; i++){
+			for(i = 1; i < 255; i++){
 				tmp[i] = (BitstreamShowBits(bs, 16) & 0xFF);
 
 				if(tmp[i] == 0)
