@@ -19,7 +19,7 @@
  *  along with this program ; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * $Id: mbcoding.c,v 1.59 2010/10/15 16:20:48 Isibaar Exp $
+ * $Id: mbcoding.c 2180 2019-11-12 14:48:35Z Isibaar $
  *
  ****************************************************************************/
 
@@ -901,8 +901,8 @@ get_mv_data(Bitstream * bs)
 
 	index -= 4;
 
-	BitstreamSkip(bs, TMNMVtab2[index].len);
-	return TMNMVtab2[index].code;
+	BitstreamSkip(bs, TMNMVtab2[index&0x7f].len);
+	return TMNMVtab2[index&0x7f].code;
 
 }
 
@@ -1965,7 +1965,8 @@ VLC const TMNMVtab2[] = {
 	{14, 10}, {14, 10}, {14, 10}, {14, 10},
 	{-14, 10}, {-14, 10}, {-14, 10}, {-14, 10},
 	{13, 10}, {13, 10}, {13, 10}, {13, 10},
-	{-13, 10}, {-13, 10}, {-13, 10}, {-13, 10}
+	{-13, 10}, {-13, 10}, {-13, 10}, {-13, 10},
+        {0, 0}, {0, 0}, {0, 0}, {0, 0}
 };
 
 short const dc_threshold[] = {
